@@ -152,18 +152,31 @@ pub struct GroupConfig {
     #[structopt(short = ".", long)]
     pub hidden: bool,
 
-    /// Does not respect .gitignore and .fdignore files.
+    /// Does not ignore files matching patterns listed in .gitignore and .fdignore.
     #[structopt(short = "A", long)]
     pub no_ignore: bool,
 
     /// Follows symbolic links.
+    ///
+    /// If this flag is set together with `--symbolic-links`, only links to
+    /// directories are followed.
     #[structopt(short = "L", long)]
     pub follow_links: bool,
 
-    /// Treats files reachable from multiple paths through
-    /// hard links as duplicates
+    /// Treats files reachable from multiple paths through links as duplicates.
+    ///
+    /// If `--symbolic-links` is not set, only hard links are matched.
+    /// If `--symbolic-links` is set, both hard and symbolic links are matched.
     #[structopt(short = "H", long)]
-    pub hard_links: bool,
+    pub match_links: bool,
+
+    /// Doesn't ignore symbolic links to files.
+    ///
+    /// Reports symbolic links, not their targets.
+    /// Symbolic links are not treated as duplicates of their targets,
+    /// unless `--match-links` is set.
+    #[structopt(short = "S", long)]
+    pub symbolic_links: bool,
 
     /// Don't count matching files found within the same directory argument as duplicates.
     #[structopt(short("I"), long, conflicts_with("follow-links"))]
@@ -185,7 +198,7 @@ pub struct GroupConfig {
     /// specified by $OUT and will read output from there.
     /// If the program modifies the original file in-place without writing to the standard output
     /// nor a distinct file, use --in-place flag.
-    #[structopt(long, value_name("command"))]
+    #[structopt(long, value_name("command"), conflicts_with("cache"))]
     pub transform: Option<String>,
 
     /// Set this flag if the command given to --transform transforms the file in-place,
@@ -370,6 +383,7 @@ impl GroupConfig {
             } else {
                 vec![]
             },
+            group_by_id: !self.match_links,
         }
     }
 
@@ -580,6 +594,24 @@ pub struct DedupeConfig {
     /// `fclones group` command, if `--isolate` option was present.
     #[structopt(long = "isolate", value_name = "path", parse(from_os_str))]
     pub isolated_roots: Vec<Path>,
+
+    /// Treats files reachable from multiple paths through links as duplicates.
+    #[structopt(short = "H", long)]
+    pub match_links: bool,
+
+    /// Doesn't lock files before performing an action on them.
+    #[structopt(long)]
+    pub no_lock: bool,
+
+    /// Allows the size of a file to be different than the size recorded during grouping.
+    ///
+    /// By default, files are checked for size to prevent accidentally removing a file
+    /// that was modified since grouping.
+    /// However, if `--transform` was used when grouping, the data sizes recorded in the `fclones group`
+    /// report likely don't match the on-disk sizes of the files. Therefore,
+    /// this flag is set automatically if `--transform` was used.
+    #[structopt(long)]
+    pub no_check_size: bool,
 }
 
 #[derive(Debug, StructOpt)]
